@@ -165,6 +165,8 @@
 	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addToFullItemArray:) withObject:dataFile withRunInMainThread:YES];
 	[addToList setDisplayStatus:NO];
 	
+	[copyFile cancelItemIfFail:addToList];
+	
 	NSString *description = [NSString stringWithFormat:@"Uploading file '%@'", [dataFile filename]];
 	[self addToQueue:copyFile withSubItems:[NSArray arrayWithObject:addToList] withDescription:description];
 	
@@ -191,6 +193,9 @@
 	NJBQueueItem *updateFileList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(searchNJBTable)];
 	[updateFileList setRunInMainThread:YES];
 	[updateFileList setDisplayStatus:NO];
+	
+	[deleteFile cancelItemIfFail:removeFromAllFiles];
+	[deleteFile cancelItemIfFail:updateFileList];
 		
 	NSString *description = [NSString stringWithFormat:@"Deleting file '%@'", [dataFile filename]];
 	[self addToQueue:deleteFile withSubItems:[NSArray arrayWithObjects:removeFromAllFiles, updateFileList, nil] withDescription:description];
@@ -250,8 +255,8 @@
 			[drawerController showDataFile:(DataFile *)[[self itemArrayDisplaying] objectAtIndex:[njbTable selectedRow]]];
 		}
 	}
-	// todo: is there a better way of finding out if it's the search field?
-	else if ([aResponder class] == [NSSearchField class])
+	// used to be [aResponder class] == [NSSearchField class]
+	else if (aResponder == njbSearchField)
 	{
 		[drawerController disableAll];
 	}
