@@ -13,9 +13,13 @@
 #import "ID3Tagger.h"
 #include <id3/tag.h>
 #include <id3/misc_support.h>
-#import "MP3Header.h"
 #import "WMATagger.h"
 #import "UnicodeWrapper.h"
+#import "MP3Length.h"
+// for stat
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 @interface ID3Tagger (PrivateAPI)
 // NB these can't be delcared in ID3Tagger.h as would then include id3/tag.h in other files which causes errors
@@ -100,16 +104,16 @@
 		return [WMATagger readTrack:filename];
 	}
 	
+	struct stat sb;
 	// check file exists
-	NSFileManager *fileManager = [[NSFileManager alloc] init];
-	// this is used below to get filesize
-	NSDictionary *fileAttributes = [fileManager fileAttributesAtPath:filename traverseLink:YES];
-	if (fileAttributes == nil)
+	if (stat([filename fileSystemRepresentation], &sb) == -1)
 		return nil;
 	
-	Track *track = [[Track alloc] init];
+	// N.B. cannot use NSFileManager as this is not thread safe
 		
-	[track setFilesize:[[fileAttributes objectForKey:NSFileSize] intValue]];
+	Track *track = [[Track alloc] init];
+	
+	[track setFilesize:sb.st_size];
 	[track setCodec:[self codec]];
 	[track setFullPath:filename];
 	
@@ -160,13 +164,7 @@
 		[track setYear:[self intFrameContentsFromTag:&myTag frameID:ID3FID_YEAR]];
 		
 		// read the length from the header
-		MP3Header *header = [[MP3Header alloc] init];
-		[track setLength:[header length:filename]];
-		[header release];
-		
-		// why can't I use id3lib like this? always returns NULL
-		/*const Mp3_Headerinfo *header = myTag.GetMp3HeaderInfo();
-		[track setLength:header->time];*/
+		[track setLength:[MP3Length length:filename]];
 		
 		myTag.Clear();
 	
@@ -179,7 +177,6 @@
 		[track setLength:[self wavFileLength:filename]];
 	}
 	
-	[fileManager release];
 	return [track autorelease];
 }
 
