@@ -7,6 +7,10 @@
 #import "NJBQueueItem.h"
 #import "defs.h"
 
+
+/* todo: ask to close app when close main window
+ */
+
 @implementation MainController
 
 // init/dealloc methods
@@ -56,6 +60,11 @@
 	return settingsTab;
 }
 
+- (DuplicateItemsTab *)duplicateItemsTab
+{
+	return duplicateItemsTab;
+}
+
 /**********************/
 
 - (void)awakeFromNib
@@ -85,6 +94,11 @@
 	}
 	
 	[status setMainThreadQueueLock:myQueueLock];
+	
+	NSNotificationCenter *nc;
+	nc = [NSNotificationCenter defaultCenter];
+	[nc addObserver:self selector:@selector(NJBConnected:) name:NOTIFICATION_CONNECTED object:nil];
+	[nc addObserver:self selector:@selector(NJBDisconnected:) name:NOTIFICATION_DISCONNECTED object:nil];
 }
 
 - (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
@@ -102,6 +116,8 @@
 	int tabIndex = [tabViewMain indexOfTabViewItem:[tabViewMain selectedTabViewItem]];
 	if (tabIndex == TAB_MUSIC)
 		[musicTab drawerWriteTag:modifiedTrack];
+	else if (tabIndex == TAB_DUPLICATES)
+		[duplicateItemsTab drawerWriteTag:modifiedTrack];
 }
 
 - (void)onFirstResponderChange:(NSResponder *)aResponder
@@ -162,17 +178,15 @@
 	[queueTab setNJBQueueConsumer:njbQueue];
 	[dataTab setNJBQueueConsumer:njbQueue];
 	[playlistsTab setNJBQueueConsumer:njbQueue];
+	[duplicateItemsTab setNJBQueueConsumer:njbQueue];
 	[settingsTab setNJBQueueConsumer:njbQueue];
 }
 
 - (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
 {
-	if (![status isIdle])
+	if (![self shouldClose])
 	{
-		int result = NSRunAlertPanel(@"Quit XNJB", @"There are still items in the queue.  Are you sure you want to quit?  This could crash your NJB and/or XNJB.",
-																 @"No", @"Yes", nil);
-		if (result == NSAlertDefaultReturn)
-			return NSTerminateCancel;
+		return NSTerminateCancel;
 	}
 	// do it in this thread as want to block
 	[theNJB disconnect];
@@ -184,6 +198,18 @@
 	return NSTerminateNow;
 }
 
+- (BOOL)shouldClose
+{
+	if (![status isIdle])
+	{
+		int result = NSRunAlertPanel(@"Quit XNJB", @"There are still items in the queue.  Are you sure you want to quit?  This could crash your NJB and/or XNJB.",
+																 @"No", @"Yes", nil);
+		if (result == NSAlertDefaultReturn)
+			return NO;
+	}
+	return YES;
+}
+
 - (id)currentTab
 {
 	return [self tabAtIndex:[tabViewMain indexOfTabViewItem:[tabViewMain selectedTabViewItem]]];
@@ -210,6 +236,9 @@
 		case TAB_SETTINGS:
 			return settingsTab;
 			break;
+		case TAB_DUPLICATES:
+			return duplicateItemsTab;
+			break;
 		case TAB_QUEUE:
 			return queueTab;
 			break;
@@ -235,9 +264,9 @@
 		if ([currentItem displayStatus])
 		{
 			if ([result class] == [NJBTransactionResult class])
-				[status taskComplete:[result success]];
+				[status taskComplete:result];
 			else
-				[status taskComplete:YES];
+				[status taskComplete:[[NJBTransactionResult alloc] initWithSuccess:YES]];
 		}
 	}
 	[myQueueLock unlock];
@@ -311,4 +340,36 @@
 	[self disconnectFromNJB];
 }
 
+- (IBAction)buttonConnect:(id)sender
+{
+	if (![theNJB isConnected])
+		[self connectToNJB];
+	else
+		[self disconnectFromNJB];
+}
+
+/* called when we disconnect
+*/
+- (void)NJBDisconnected:(NSNotification *)note
+{
+	[connectButton setTitle:@"Connect"];
+}
+
+/* called when we connect
+*/
+- (void)NJBConnected:(NSNotification *)note
+{
+	[connectButton setTitle:@"Disconnect"];
+}
+
+/* called when the main window is closing
+ */
+- (BOOL)windowShouldClose:(id)sender
+{
+	[NSApp terminate:sender];
+	// don't close the window, because if we said don't close the app, we don't want the window closed but app still running
+	// if user asked to terminate app rather than just close window, windowShouldClose: is not called so we will close anyway
+	return NO;
+}
+
 @end
