@@ -15,6 +15,7 @@
 	int previousTrackIndexSelected;
 	NSDictionary *fileExtensionsToCodecsDictionary;
 	activeObjects objectToSaveTrack;
+	BOOL trackListUpToDate;
 }
 - (void)loadTracks;
 - (NJBTransactionResult *)downloadTrackList;
@@ -22,5 +23,6 @@
 - (void)copyTrackFromNJB:(Track *)track;
 - (void)replaceTrack:(Track *)replace withTrack:(Track *)new;
 - (void)writeTagToFile:(Track *)track;
+- (NJBTransactionResult *)uploadTrack:(Track *)track;
 
 @end
