@@ -59,6 +59,7 @@
 	[browser setAllowedExtensions:extensions];
 	[browser setBaseLocation:[preferences musicTabDir]];
 	
+	[njbTable setAllowedExtensions:extensions];
 	[njbTable setAutosaveName:PREF_KEY_MUSIC_TAB_TABLE];
 	[njbTable setAutosaveTableColumns:YES];
 }
@@ -66,16 +67,7 @@
 - (void)onConnectAndActive
 {
 	[super onConnectAndActive];
-	if ([playlistsTab fullItemArray])
-	{
-		// playlistsTab has already got it
-		[fullItemArray release];
-		fullItemArray = [[MyMutableArray alloc] initWithMyMutableArray:[playlistsTab fullItemArray]];
-		[self clearSortingArrows];
-		[self searchNJBTable];
-	}
-	else
-		[self loadTracks];
+	[self loadTracks];
 }
 
 - (SEL)sortSelectorForColumn:(NSString *)columnID
@@ -227,6 +219,7 @@
 {
 	if (![theNJB isConnected])
 		return;
+		
 	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(downloadTrack:) withObject:track];
 	[copyTrack setStatus:STATUS_DOWNLOADING_TRACK];
 	NSString *description = [NSString stringWithFormat:@"Downloading track '%@'", [track title]];
@@ -242,9 +235,11 @@
 	if ([preferences writeTagAfterCopy] && [track codec] == CODEC_MP3)
 	{
 		NJBQueueItem *writeTag = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(writeTagToFile:) withObject:track];
-		[writeTag setRunInMainThread:NO];
+		// putting this to NO causes weird and wonderful crashes later on e.g. memory of the njb_t struct in theNJB getting overwritten...
+		// should be YES though (wasn't pre 0.23)
+		[writeTag setRunInMainThread:YES];
 		[writeTag setDisplayStatus:NO];
-		
+
 		[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:updatePlaylistsTab, writeTag, nil] withDescription:description];
 		[writeTag release];
 	}
@@ -256,17 +251,14 @@
 }
 
 /* copy the file at filename to
- * the NJB
+ * the NJB.
  */
 - (void)copyFileToNJB:(NSString *)filename
 {
-	ID3Tagger *tagger = [[ID3Tagger alloc] init];
-	[tagger setFileExtensionsToCodecsDictionary:fileExtensionsToCodecsDictionary];
-	[tagger setFilename:filename];
-	Track *track = [tagger readTrack];
+	Track *track = [[Track alloc] init];
 	[track setFullPath:filename];
 	[self copyTrackToNJB:track];
-	[tagger release];
+	[track release];
 }
 
 /* actually add the track to the queue
@@ -276,30 +268,32 @@
 {
 	if (![theNJB isConnected])
 		return;
-	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(uploadTrack:) withObject:track];
+	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(uploadTrack:) withObject:track];
 	[copyTrack setStatus:STATUS_UPLOADING_TRACK];
 	
-	NJBQueueItem *addToTrackList = [[NJBQueueItem alloc] initWithTarget:fullItemArray withSelector:@selector(addObject:) withObject:track withRunInMainThread:YES];
-	[addToTrackList setDisplayStatus:NO];
+	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addToFullItemArray:) withObject:track withRunInMainThread:YES];
+	[addToList setDisplayStatus:NO];
 	
-	// call searchNJBTable to pretend the search list changed to update
-	// saves data and does a reload
-	NJBQueueItem *updateTrackList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(searchNJBTable)];
-	[updateTrackList setRunInMainThread:YES];
-	[updateTrackList setDisplayStatus:NO];
+	[copyTrack cancelItemIfFail:addToList];
 	
-	// tell PlaylistsTab
-	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
-	[updatePlaylistsTab setRunInMainThread:NO];
-	[updatePlaylistsTab setDisplayStatus:NO];
-	
-	NSString *description = [NSString stringWithFormat:@"Uploading track '%@'", [track title]];
-	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToTrackList, updateTrackList, updatePlaylistsTab, nil] withDescription:description];
+	NSString *description = [NSString stringWithFormat:@"Uploading track '%@'", [track fullPath]];
+	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToList, nil] withDescription:description];
 	
 	[copyTrack release];
-	[addToTrackList release];
-	[updateTrackList release];
-	[updatePlaylistsTab release];
+	[addToList release];
+}
+
+/* uploads the track to the jukebox, run in worker thread
+ */
+- (NJBTransactionResult *)uploadTrack:(Track *)track
+{
+	ID3Tagger *tagger = [[ID3Tagger alloc] init];
+	[tagger setFileExtensionsToCodecsDictionary:fileExtensionsToCodecsDictionary];
+	[tagger setFilename:[track fullPath]];
+	Track *trackWithTag = [tagger readTrack];
+	[track setValuesToTrack:trackWithTag];
+	[tagger release];
+	return [theNJB uploadTrack:track];
 }
 
 /* write the info in track to the tag in the file
@@ -388,11 +382,12 @@
 			if (rowIndex == -1)
 				return;	
 			
-			NJBQueueItem *changeTrackTag = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(changeTrackTag:) withObject:modifiedTrack];
-			[changeTrackTag setStatus:STATUS_UPDATING_TRACK_TAG];
-			
 			Track *oldTrack = (Track *)[[self itemArrayDisplaying] objectAtIndex:rowIndex];
 			
+			NJBQueueItem *changeTrackTag = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(changeTrackTagTo:from:) withObject:modifiedTrack];
+			[changeTrackTag setObject2:oldTrack];
+			[changeTrackTag setStatus:STATUS_UPDATING_TRACK_TAG];
+			
 			NJBQueueItem *updateTrackList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(replaceTrack:withTrack:)
 																																withObject:oldTrack withRunInMainThread:YES];
 			[updateTrackList setObject2:modifiedTrack];
@@ -404,18 +399,15 @@
 			[updateDisplay setRunInMainThread:YES];
 			[updateDisplay setDisplayStatus:NO];
 			
-			// tell PlaylistsTab
-			NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
-			[updatePlaylistsTab setRunInMainThread:NO];
-			[updatePlaylistsTab setDisplayStatus:NO];
+			[updateTrackList cancelItemIfFail:updateTrackList];
+			[updateTrackList cancelItemIfFail:updateDisplay];
 			
 			NSString *description = [NSString stringWithFormat:@"Changing tag for track '%@'", [oldTrack title]];
-			[self addToQueue:changeTrackTag withSubItems:[NSArray arrayWithObjects:updateTrackList, updateDisplay, updatePlaylistsTab, nil] withDescription:description];
+			[self addToQueue:changeTrackTag withSubItems:[NSArray arrayWithObjects:updateTrackList, updateDisplay, nil] withDescription:description];
 
 			[changeTrackTag release];
 			[updateTrackList release];
 			[updateDisplay release];
-			[updatePlaylistsTab release];
 			
 			break;
 		case OBJECT_FILE_BROWSER:
@@ -436,17 +428,15 @@
 	// release old track list
 	[truncatedItemArray release];
 	
-	if ([searchString length] == 0)
+	if ([searchString length] != 0)
 	{
-		truncatedItemArray = nil;
-		[njbTable reloadData];
-		return;
+		truncatedItemArray = [fullItemArray searchFor:searchString];
+		[truncatedItemArray retain];
 	}
-	
-	truncatedItemArray = [fullItemArray searchFor:searchString];
-	[truncatedItemArray retain];
-	
+	else
+		truncatedItemArray = nil;
 	[njbTable reloadData];
+	
 	if ([njbTable selectedRow] != -1)
 	{
 		// update display to show new track
@@ -500,7 +490,8 @@
 	}
 	else if ([aResponder class] == [NSMatrix class])
 		objectToSaveTrack = OBJECT_FILE_BROWSER;
-	else if ([aResponder class] == [NSSearchField class])
+	// used to be [aResponder class] == [NSSearchField class]
+	else if (aResponder == njbSearchField)
 	{
 		[drawerController disableAll];
 	}
@@ -525,11 +516,15 @@
 																							 withRunInMainThread:YES];
 	[updateTable setDisplayStatus:NO];
 	
+	[getTracks cancelItemIfFail:updateTable];
+	
 	NSString *description = @"Getting track list";
 	[self addToQueue:getTracks withSubItems:[NSArray arrayWithObjects:clearSortingArrows, updateTable, nil] withDescription:description];
 	[getTracks release];
 	[clearSortingArrows release];
 	[updateTable release];
+	// it's not actually yet, but will be in a while, don't want to go copying anything
+	trackListUpToDate = YES;
 }
 
 /* this is called from the queue consumer 
@@ -546,6 +541,8 @@
 	
 	[fullItemArray retain];
 	
+	trackListUpToDate = YES;
+	
 	return [result autorelease];
 }
 
@@ -554,7 +551,7 @@
 	Track *track = (Track *)item;
 
 	NJBQueueItem *deleteTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(deleteTrack:)
-																												withObject:([NSNumber numberWithInt:[track trackID]])];
+																												withObject:track];
 	[deleteTrack setStatus:STATUS_DELETING_TRACK];
 		
 	NJBQueueItem *removeFromAllTracks = [[NJBQueueItem alloc] initWithTarget:fullItemArray withSelector:@selector(removeObject:)
@@ -568,18 +565,15 @@
 	[updateTrackList setRunInMainThread:YES];
 	[updateTrackList setDisplayStatus:NO];
 	
-	// tell PlaylistsTab
-	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
-	[updatePlaylistsTab setRunInMainThread:NO];
-	[updatePlaylistsTab setDisplayStatus:NO];
-		
+	[deleteTrack cancelItemIfFail:removeFromAllTracks];
+	[deleteTrack cancelItemIfFail:updateTrackList];
+	
 	NSString *description = [NSString stringWithFormat:@"Deleting track '%@'", [track title]];
-	[self addToQueue:deleteTrack withSubItems:[NSArray arrayWithObjects:removeFromAllTracks, updateTrackList, updatePlaylistsTab, nil] withDescription:description];
-		
+	[self addToQueue:deleteTrack withSubItems:[NSArray arrayWithObjects:removeFromAllTracks, updateTrackList, nil] withDescription:description];
+
 	[deleteTrack release];
 	[removeFromAllTracks release];
 	[updateTrackList release];
-	[updatePlaylistsTab release];
 		
 	NSLog(@"Deleting track id %d with title %@", [track trackID], [track title]);		
 }
@@ -600,6 +594,15 @@
 - (void)activate
 {
 	[super activate];
+	if (!trackListUpToDate)
+	{
+		[fullItemArray release];
+		fullItemArray = nil;
+		[truncatedItemArray release];
+		truncatedItemArray = nil;
+		[njbTable reloadData];
+		[self loadTracks];
+	}	
 	if (activeObject == OBJECT_NJB_TABLE)
 	{
 		if ([njbTable selectedRow] != -1)
@@ -628,4 +631,12 @@
 	[super applicationTerminating:note];
 }
 
-@end
+- (void)NJBTrackListModified:(NSNotification *)note
+{
+	if (!isActive)
+	{
+		trackListUpToDate = NO;
+	}
+}
+
+@end
\ No newline at end of file
