@@ -43,6 +43,10 @@
 	[browser setTarget:self];
 	[browser setFileAction:@selector(fileSystemBrowserFileClick:)];
 	[browser setDirectoryAction:@selector(fileSystemBrowserDirectoryClick:)];
+	
+	// allow drag & drop
+	[njbTable registerDragAndDrop];
+	[njbTable disallowCopies];
 }
 
 /* makes a string into a legal filename
@@ -186,6 +190,7 @@
 	[deleteFromActiveObjectButton setEnabled:[self canDelete]];
 	[copyToNJBButton setEnabled:[self canCopyToNJB]];
 	[copyFromNJBButton setEnabled:[self canCopyFromNJB]];
+	[njbTable allowCopies];
 }
 
 /* disable transfer buttons
@@ -195,6 +200,7 @@
 	[deleteFromActiveObjectButton setEnabled:[self canDelete]];
 	[copyToNJBButton setEnabled:NO];
 	[copyFromNJBButton setEnabled:NO];
+	[njbTable disallowCopies];
 }
 
 /* get a list of all the files to copy to the NJB
@@ -290,7 +296,7 @@
 	[selectedItems release];
 }
 
-/* delete an off the NJB, to be implemented by superclass
+/* delete an item off the NJB, to be implemented by superclass
  */
 - (void)deleteFromNJB:(MyItem *)item{}
 
