@@ -10,7 +10,7 @@
 
 @interface MyTableTab : MyTab {
   IBOutlet NSTableView *njbTable;
-	IBOutlet NSSearchField *njbSearchField;
+	IBOutlet NSTextField *njbSearchField;
 	NSDictionary *sortSelectors;
 	NSString *currentColumnID;
 	sortDirections currentSortDirection; 
