@@ -71,19 +71,25 @@
 - (void)onConnectAndActive
 {
 	[super onConnectAndActive];
-	if ([musicTab fullItemArray])
-	{
-		// musicTab has already got it
-		[fullItemArray release];
-		fullItemArray = [[MyMutableArray alloc] initWithMyMutableArray:[musicTab fullItemArray]];
-		[self clearSortingArrows];
-		[self searchNJBTable];
-	}
-	else
-		[self loadTracks];
+	[self loadTracks];
 	[self loadPlaylists];
 }
 
+- (void)activate
+{
+	// this must be called first to run onConnectAndActive so we can download the tracks there
+	[super activate];
+	if (!trackListUpToDate)
+	{
+		[fullItemArray release];
+		fullItemArray = nil;
+		[truncatedItemArray release];
+		truncatedItemArray = nil;
+		[njbTable reloadData];
+		[self loadTracks];
+	}
+}
+
 - (void)awakeFromNib
 {
 	[super awakeFromNib];
@@ -244,6 +250,8 @@
 	[getTracks release];
 	[clearSortingArrows release];
 	[updateTable release];
+	// it's not actually yet, but will be in a while, don't want to go copying anything
+	trackListUpToDate = YES;
 }
 
 /* this is called from the queue consumer 
@@ -260,6 +268,8 @@
 	
 	[fullItemArray retain];
 	
+	trackListUpToDate = YES;
+	
 	return [result autorelease];
 }
 
@@ -445,6 +455,7 @@
 	NSLog(@"Updating playlist from deletePlaylist:");
 	[self deleteJukeboxPlaylist:(Playlist *)[playlists objectAtIndex:row]];
 	[playlists removeObjectAtIndex:row];
+	tracksInCurrentPlaylist = nil;
 	
 	[playlistsTable reloadData];
 }
@@ -668,4 +679,12 @@
 	[playlistsTrackTable selectRow:(index + 1) byExtendingSelection:YES];
 }
 
+- (void)NJBTrackListModified:(NSNotification *)note
+{
+	if (!isActive)
+	{
+		trackListUpToDate = NO;
+	}
+}
+
 @end
