@@ -1,3 +1,10 @@
+/**
+ * \file protocol.c
+ *
+ * This file contains the major parts of the NJB1 (OaSIS) protocol
+ * for all jukebox functionality.
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -14,6 +21,10 @@
 #include "defs.h"
 #include "base.h"
 #include "eax.h"
+#include "songid.h"
+#include "datafile.h"
+#include "njbtime.h"
+#include "playlist.h"
 
 extern int __sub_depth;
 
@@ -21,6 +32,9 @@ extern int __sub_depth;
 /**
  * Initializes the basic state of the njb->protocol_state for the
  * NJB1-device.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @return 0 on success, -1 on failure.
  */
 int njb_init_state (njb_t *njb) {
   __dsub= "njb_init_state";
@@ -39,6 +53,9 @@ int njb_init_state (njb_t *njb) {
   state->libcount = 0;
   state->first_eax = NULL;
   state->next_eax = NULL;
+  state->reset_get_track_tag = 0;
+  state->reset_get_playlist = 0;
+  state->reset_get_datafile_tag = 0;
   njb->protocol_state = (unsigned char *) state;
 
   __leave;
@@ -54,6 +71,94 @@ int njb_init_state (njb_t *njb) {
  * what it means, but I have implemented it where I've run into it.
  */
 
+
+/**
+ * This returns the string representing an internal NJB1 error code.
+ */
+#define NJB_STATUS(a,b) njb_error_add_string(a,subroutinename,njb_status_string(b))
+static const char *njb_status_string (unsigned char code)
+{
+  /* This is not very thread safe... */
+  static char buffer[100];
+  
+  switch (code)
+    {
+    case NJB_MSG_OKAY:
+      return "no error";
+      
+    case NJB_ERR_FAILED:
+      return "operation failed";
+    case NJB_ERR_DEVICE_BUSY:
+      return "device busy";
+    case NJB_ERR_STORAGE_FULL:
+      return "storage full";
+    case NJB_ERR_HD_GENERAL_ERROR:
+      return "general hard drive failure";
+    case NJB_ERR_SETTIME_REJECTED:
+      return "set time rejected";
+      
+    case NJB_ERR_TRACK_NOT_FOUND:
+      return "track not found";
+    case NJB_ERR_TRACK_EXISTS:
+      return "track exists";
+    case NJB_ERR_TITLE_MISSING:
+      return "title missing";
+    case NJB_ERR_CODEC_MISSING:
+      return "CODEC missing";
+    case NJB_ERR_SIZE_MISSING:
+      return "size missing";
+    case NJB_ERR_IO_OPERATION_ABORTED:
+      return "I/O operation aborted";
+    case NJB_ERR_READ_WRITE_ERROR:
+      return "read/write error";
+    case NJB_ERR_NOT_OPENED:
+      return "not opened";
+    case NJB_ERR_UPLOAD_DENIED:
+      return "upload denied";
+      
+    case NJB_ERR_PLAYLIST_NOT_FOUND:
+      return "playlist not found";
+    case NJB_ERR_PLAYLIST_EXISTS:
+      return "playlist exists";
+    case NJB_ERR_PLAYLIST_ITEM_NOT_FOUND:
+      return "playlist item not found";
+    case NJB_ERR_PLAYLIST_ITEM_EXISTS:
+      return "playlist item exists";
+
+    case NJB_MSG_QUEUED_AUDIO_STARTED:
+      return "queued audio started";
+    case NJB_MSG_PLAYER_FINISHED:
+      return "player finished";
+    case NJB_ERR_VOLUME_CONTROL_UNAVAILABLE:
+      return "volume control unavailable";
+    case NJB_ERR_QUEUE_ALREADY_EMPTY:
+      return "queue already empty";
+      
+    case NJB_ERR_DATA_NOT_FOUND:
+      return "data not found";
+    case NJB_ERR_DATA_NOT_OPENED:
+      return "data not opened";
+      
+    case NJB_ERR_UNDEFINED_ERR:
+      return "undefined error";
+      
+    case NJB_ERR_NOT_IMPLEMENTED:
+      return "function not implemented";
+    }
+  
+  sprintf(buffer, "unknown error %02X", code);
+  return buffer;
+}
+
+/**
+ * This function sets the library counter on the device.
+ * The library counter is sent back and forth to the device to indicate if
+ * something in the library has changed.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @param count is the current library counter to set on the device.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_set_library_counter (njb_t *njb, u_int64_t count)
 {
 	__dsub= "njb_set_library_counter";
@@ -67,7 +172,7 @@ int njb_set_library_counter (njb_t *njb, u_int64_t count)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SET_LIBRARY_COUNTER, 0, 0, 8, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -79,6 +184,15 @@ int njb_set_library_counter (njb_t *njb, u_int64_t count)
 	return 0;
 }
 
+/**
+ * This function gets the library counter from the device.
+ * The library counter is sent back and forth to the device to indicate if
+ * something in the library as changed.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @lcount is a pointer to the current library counter to get from the device.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_get_library_counter (njb_t *njb, njblibctr_t *lcount)
 {
 	__dsub= "njb_get_library_counter";
@@ -91,13 +205,13 @@ int njb_get_library_counter (njb_t *njb, njblibctr_t *lcount)
 	
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_LIBRARY_COUNTER, 0, 0, 25, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] & 0xf ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	} else if ( data[0] ) {
@@ -116,11 +230,20 @@ int njb_get_library_counter (njb_t *njb, njblibctr_t *lcount)
 	return 0;
 }
 
-int njb_ping (njb_t *njb, njbid_t *njbid)
+/**
+ * This function pings the NJB1 device to see if it is up and running,
+ * and ready for action. This is done every now and then.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @param njbid is a jukebox ID type to fill in with information.
+ * @return 0 on success, -1 on failure.
+ */
+int njb_ping (njb_t *njb)
 {
 	__dsub= "njb_ping";
 	ssize_t bread;
 	unsigned char data[58];
+	njbid_t *njbid = njb->njbid;
 
 	__enter;
 	
@@ -128,34 +251,33 @@ int njb_ping (njb_t *njb, njbid_t *njbid)
 
        	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_PING, 0, 0, 0, NULL) == -1 ) {
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        		__leave;
        		return -1;
        	}
 
        	if ( (bread= usb_pipe_read(njb, data, 58)) == -1 ) {
-       		NJB_ERROR(EO_USBBLK);
+       		NJB_ERROR(njb, EO_USBBLK);
        		__leave;
        		return -1;
        	} else if ( bread < 58 ) {
-       		NJB_ERROR(EO_RDSHORT);
+       		NJB_ERROR(njb, EO_RDSHORT);
        		__leave;
        	       	return -1;
        	}
 
        	if ( data[0] ) {
-       		NJB_STATUS(data[0]);
+       		NJB_STATUS(njb,data[0]);
        		__leave;
        		return -1;
        	}
        	memcpy(&njbid->id, &data[1], 16);
-       	njbid->fwRel= (u_int8_t) 0;
-       	njbid->fwMinor= (u_int8_t) data[19];
-       	njbid->fwMajor= (u_int8_t) data[20];
-	/* Copy this -- eventually move it into njb_t */
-	njb->fwRel = njbid->fwRel;
-	njb->fwMinor = njbid->fwMinor;
-	njb->fwMajor = njbid->fwMajor;
+       	njbid->fwRel = (u_int8_t) 0;
+       	njbid->fwMinor = (u_int8_t) data[19];
+       	njbid->fwMajor = (u_int8_t) data[20];
+	njbid->hwRel = (u_int8_t) 0;
+	njbid->hwMinor = (u_int8_t) 0;
+	njbid->hwMajor = (u_int8_t) 1;
        	memcpy(&njbid->productName, &data[25], 32);
        	njbid->power= (u_int8_t) data[57];
 
@@ -170,6 +292,12 @@ int njb_ping (njb_t *njb, njbid_t *njbid)
 	return 0;
 }
 
+/**
+ * This function verifies if the last command was successful or not.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_verify_last_command (njb_t *njb)
 {
 	__dsub= "njb_verify_last_command";
@@ -180,13 +308,13 @@ int njb_verify_last_command (njb_t *njb)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_VERIFY_LAST_CMD, 0, 0, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -195,6 +323,12 @@ int njb_verify_last_command (njb_t *njb)
 	return 0;
 }
 
+/**
+ * This function captures the NJB1 device.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_capture (njb_t *njb, int which)
 {
 	__dsub= "njb_capture";
@@ -208,13 +342,13 @@ int njb_capture (njb_t *njb, int which)
        	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
        		mode, 0, 0, 1, &data) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        		__leave;
        		return -1;
        	}
 
        	if ( data && (data & 0xf) ) {
-       		NJB_ERROR(EO_BADSTATUS);
+       		NJB_ERROR(njb, EO_BADSTATUS);
        		__leave;
        		return -1;
        	}
@@ -223,6 +357,12 @@ int njb_capture (njb_t *njb, int which)
 	return 0;
 }
 
+/**
+ * Get a track header from the NJB1.
+ *
+ * @return 0 on OK, -1 on error, -2 if the last header has been
+ *         retrieved.
+ */
 int njb_get_track_tag_header (njb_t *njb, njbttaghdr_t *tagh, int cmd)
 {
 	__dsub= "njb_get_track_tag_header";
@@ -235,17 +375,21 @@ int njb_get_track_tag_header (njb_t *njb, njbttaghdr_t *tagh, int cmd)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] == NJB_ERR_TRACK_NOT_FOUND ) {
-		NJB_ERROR(EO_EOM);
+	        /*
+	         * Ignore this "error" as of now, it probably
+		 * just mean "end of transmission" or "zero tags left". 
+		 */
+	        /* NJB_ERROR(njb, EO_EOM); return -1; */
 		__leave;
-		return -1;
+		return -2;
 	} else if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -257,20 +401,20 @@ int njb_get_track_tag_header (njb_t *njb, njbttaghdr_t *tagh, int cmd)
 	return 0;
 }
 
-songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
+njb_songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
 {
 	__dsub= "njb_get_track_tag";
 	u_int16_t msw, lsw;
 	ssize_t bread;
 	void *data;
-	songid_t *song;
+	njb_songid_t *song;
 	unsigned char *dp;
 
 	__enter;
 
 	data= malloc(tagh->size+5);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -281,14 +425,14 @@ songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_GET_TRACK_TAG, msw, lsw, 0, NULL) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
 	if ( (bread= usb_pipe_read(njb, data, tagh->size+5)) == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return NULL;
@@ -305,6 +449,12 @@ songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
 	return song;
 }
 
+/**
+ * Gets a playlist header.
+ *
+ * @return 0 on OK, -1 on error, -2 means "retry", -3 means
+ *         "not found" or "last playlist already retrieved".
+ */
 int njb_get_playlist_header (njb_t *njb, njbplhdr_t *plh, int cmd)
 {
 	__dsub= "njb_get_playlist_header";
@@ -317,23 +467,26 @@ int njb_get_playlist_header (njb_t *njb, njbplhdr_t *plh, int cmd)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] == NJB_ERR_PLAYLIST_NOT_FOUND ) {
-		NJB_ERROR(EO_EOM);
+	        /*
+	         * Ignore this "error" as of now, it probably
+		 * just mean "end of transmission" or "zero tags left". 
+		 */
+	        /* NJB_ERROR(njb, EO_EOM); return -1; */
 		__leave;
-		return -1;
+		return -3;
 	} else if ( (data[0]) & 0xf ) { 
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	} else if ( data[0] ) {
-		NJB_ERROR(EO_AGAIN);
 		__leave;
-		return -1;
+		return -2;
 	}
 
 	plh->plid = njb1_bytes_to_32bit(&data[1]);
@@ -343,20 +496,20 @@ int njb_get_playlist_header (njb_t *njb, njbplhdr_t *plh, int cmd)
 	return 0;
 }
 
-playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
+njb_playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
 {
 	__dsub= "njb_get_playlist";
 	u_int16_t msw, lsw;
 	ssize_t bread;
 	void *data;
-	playlist_t *pl;
+	njb_playlist_t *pl;
 	unsigned char *dp;
 
 	__enter;
 
-	data= malloc(plh->size+5);
+	data = malloc(plh->size+5);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_EOM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -366,29 +519,28 @@ playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
 	msw= get_msw(plh->plid);
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
-		NJB_CMD_GET_PLAYLIST, msw, lsw, 0, NULL) == -1 ) {
-
-		NJB_ERROR(EO_USBCTL);
+	     NJB_CMD_GET_PLAYLIST, msw, lsw, 0, NULL) == -1 ) {
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
 	if ( (bread= usb_pipe_read(njb, data, plh->size+5)) == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return NULL;
 	} else if ( bread < plh->size+5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
-	dp= (unsigned char *) data;
+	dp = (unsigned char *) data;
 
-	pl= playlist_unpack(&dp[5], plh->size);
+	pl = playlist_unpack(&dp[5], plh->size);
 
 	free(data);
 	
@@ -396,6 +548,11 @@ playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
 	return pl;
 }
 
+/**
+ * Get disk usage for the NJB1
+ *
+ * @return 0 on OK, -1 on failure, -2 means "try again"
+ */
 int njb_get_disk_usage (njb_t *njb, u_int64_t *total, u_int64_t *free)
 {
 	__dsub= "njb_get_disk_usage";
@@ -408,19 +565,18 @@ int njb_get_disk_usage (njb_t *njb, u_int64_t *total, u_int64_t *free)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_DISK_USAGE, 0, 0, 17, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] & 0xf ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	} else if ( data[0] ) {
-		NJB_ERROR(EO_AGAIN);
 		__leave;
-		return -1;
+		return -2;
 	}
 
 	*total = njb1_bytes_to_64bit(&data[1]);
@@ -442,7 +598,7 @@ int njb_get_owner_string (njb_t *njb, owner_string name)
        	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_GET_OWNER_STRING, 0, 0, 0, NULL) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        		__leave;
        		return -1;
        	}
@@ -450,16 +606,16 @@ int njb_get_owner_string (njb_t *njb, owner_string name)
        	if ( (bread= usb_pipe_read(njb, data, OWNER_STRING_LENGTH+1))
        		== -1 ) {
 
-       		NJB_ERROR(EO_USBBLK);
+       		NJB_ERROR(njb, EO_USBBLK);
        		__leave;
        		return -1;
        	} else if ( bread < OWNER_STRING_LENGTH+1 ) {
-       		NJB_ERROR(EO_RDSHORT);
+       		NJB_ERROR(njb, EO_RDSHORT);
        		__leave;
        		return -1;
        	}
        	if ( data[0] ) {
-       		NJB_STATUS((unsigned char) data[0]);
+       		NJB_STATUS(njb,(unsigned char) data[0]);
        		__leave;
        		return -1;
        	}
@@ -481,7 +637,7 @@ int njb_set_owner_string (njb_t *njb, owner_string name)
        	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_SET_OWNER_STRING, 0, 0, 0, NULL) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        		__leave;
        		return -1;
        	}
@@ -489,11 +645,11 @@ int njb_set_owner_string (njb_t *njb, owner_string name)
        	if ( (bwritten= usb_pipe_write(njb, name, OWNER_STRING_LENGTH))
        		== -1 ) {
 
-       		NJB_ERROR(EO_USBBLK);
+       		NJB_ERROR(njb, EO_USBBLK);
        		__leave;
        		return -1;
        	} else if ( bwritten < OWNER_STRING_LENGTH ) {
-       		NJB_ERROR(EO_WRSHORT);
+       		NJB_ERROR(njb, EO_WRSHORT);
        		__leave;
        		return -1;
        	}
@@ -503,6 +659,12 @@ int njb_set_owner_string (njb_t *njb, owner_string name)
 	return 0;
 }
 
+/**
+ * Retrieves a datafile header from the NJB1
+ *
+ * @return 0 on OK, -1 on error, -2 means the last header
+ *         has already been returned.
+ */
 int njb_get_datafile_header (njb_t *njb, njbdfhdr_t *dfh, int cmd)
 {
 	__dsub= "njb_get_datafile_header";
@@ -515,17 +677,21 @@ int njb_get_datafile_header (njb_t *njb, njbdfhdr_t *dfh, int cmd)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return 0;
 	}
 
-	if ( data[0] == 0x60 ) {
-		NJB_ERROR(EO_EOM);
+	if ( data[0] == NJB_ERR_DATA_NOT_FOUND ) {
+	        /*
+	         * Ignore this "error" as of now, it probably
+		 * just mean "end of transmission" or "zero tags left". 
+		 */
+	        /* NJB_ERROR(njb, EO_EOM); return -1; */
 		__leave;
-		return -1;
+		return -2;
 	} else if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -537,20 +703,20 @@ int njb_get_datafile_header (njb_t *njb, njbdfhdr_t *dfh, int cmd)
 	return 0;
 }
 
-datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
+njb_datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
 {
 	__dsub= "njb_get_datafile_tag";
 	u_int16_t msw, lsw;
 	ssize_t bread;
 	void *data;
-	datafile_t *df;
+	njb_datafile_t *df;
 	unsigned char *dp;
 
 	__enter;
 
 	data= malloc(dfh->size+5);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -562,19 +728,19 @@ datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_GET_DATAFILE_TAG, msw, lsw, 0, NULL) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
 	if ( (bread= usb_pipe_read(njb, data, dfh->size+5)) == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return NULL;
 	} else if ( bread < dfh->size+5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		free(data);
 		__leave;
 		return NULL;
@@ -615,7 +781,7 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 
 	/* Too large transfer requested */
 	if ( bsize > NJB_XFER_BLOCK_SIZE ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return -1;
 	}
@@ -631,14 +797,14 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		       NJB_CMD_RECEIVE_FILE_BLOCK, 0, 0, 8, filedata) == -1 ) {
 	  
-	  NJB_ERROR(EO_USBCTL);
+	  NJB_ERROR(njb, EO_USBCTL);
 	  __leave;
 	  return -1;
 	}
 
 	/* Read the status and block header first */
 	if ( (bread = usb_pipe_read(njb, bp, bsize + NJB_XFER_BLOCK_HEADER_SIZE)) == -1 ) {
-	  NJB_ERROR(EO_USBBLK);
+	  NJB_ERROR(njb, EO_USBBLK);
 	  __leave;
 	  return -1;
 	} else if ( bread < bsize + NJB_XFER_BLOCK_HEADER_SIZE ) {
@@ -649,7 +815,7 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 	/* Check status in header, should it be if (!lastshort)??? */
 	status = ((unsigned char *)bp)[0];
 	if (status) {
-	  NJB_STATUS(status);
+	  NJB_STATUS(njb,status);
 	  __leave;
 	  return -1;
 	}
@@ -670,7 +836,7 @@ int njb_request_file (njb_t *njb, u_int32_t fileid)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_REQUEST_TRACK, 1, 0, 4, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -689,13 +855,13 @@ int njb_transfer_complete (njb_t *njb)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_TRANSFER_COMPLETE, 0, 0, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data && (data & 0xf) ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -721,35 +887,35 @@ int njb_send_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SEND_TRACK_TAG, 0, 0, 4, utagsz) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
-	bwritten= usb_pipe_write(njb, tag, tagh->size);
+	bwritten = usb_pipe_write(njb, tag, tagh->size);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < tagh->size ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
 
-	bread= usb_pipe_read(njb, data, 5);
+	bread = usb_pipe_read(njb, data, 5);
 	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bread < 5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -773,12 +939,13 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 	memset(utagsz, 0, 4);
 	memset(data, 0, 5);
 
-	from_32bit_to_njb1_bytes(dfh->size, &utagsz[0]);
+	/* Add five here for the header overhead */
+	from_32bit_to_njb1_bytes(dfh->size+5, &utagsz[0]);
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SEND_DATAFILE_TAG, 0, 0, 4, utagsz) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -788,7 +955,7 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 
 	padded_tag = (unsigned char *) malloc(dfh->size+5);
 	if ( padded_tag == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -801,12 +968,12 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 	bwritten= usb_pipe_write(njb, padded_tag, dfh->size+5);
 	if ( bwritten == -1 ) {
 		free(padded_tag);
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < dfh->size+5 ) {
 		free(padded_tag);
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
@@ -815,19 +982,19 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 
 	/* Read back datafile ID */
 
-	bread= usb_pipe_read(njb, data, 5);
+	bread = usb_pipe_read(njb, data, 5);
 	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bread < 5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -857,7 +1024,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 	__enter;
 
 	if ( blocksize > NJB_XFER_BLOCK_SIZE ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return -1;
 	}
@@ -875,7 +1042,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 			 NJB_CMD_SEND_FILE_BLOCK, lsw, msw, 1, &status)
 	       == -1 ) {
 	    
-	    NJB_ERROR(EO_USBCTL);
+	    NJB_ERROR(njb, EO_USBCTL);
 	    __leave;
 	    return -1;
 	  }
@@ -897,7 +1064,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 #endif
 	    retries --;
 	    if ( !retries ) {
-	      NJB_ERROR(EO_BADSTATUS);
+	      NJB_ERROR(njb, EO_BADSTATUS);
 	      __leave;
 	      return -1;
 	    }
@@ -907,7 +1074,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 	bwritten = usb_pipe_write(njb, data, blocksize);
 
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	}
@@ -926,13 +1093,13 @@ int njb_stop_play (njb_t *njb)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_STOP_PLAY, 0, 0, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -953,13 +1120,13 @@ int njb_get_eax_size (njb_t *njb, u_int32_t *size)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_EAX_SIZE, 0, 0, 5, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] ) {
-		NJB_ERROR(EO_BADSTATUS);
+		NJB_ERROR(njb, EO_BADSTATUS);
 		__leave;
 		return -1;
 	}
@@ -970,71 +1137,15 @@ int njb_get_eax_size (njb_t *njb, u_int32_t *size)
 	return 0;
 }
 
-/* This is an old, deprecated routine */
-eax_t *njb_get_eax (njb_t *njb, u_int32_t size)
-{
-	__dsub= "njb_get_eax";
-	eax_t *eax= NULL;
-	char *data;
-	ssize_t bread;
-	u_int32_t actsize;
-	unsigned char usize[4];
-
-	__enter;
-
-	data= (char *) malloc(size);
-	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
-		__leave;
-		return NULL;
-	}
-	memset(data, 0, size);
-	memset(usize, 0, 4);
-	
-	from_32bit_to_njb1_bytes(size, &usize[0]);
-
-	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, NJB_CMD_GET_EAX,
-		0, 0, 4, usize) == -1 ) {
-
-		NJB_ERROR(EO_USBCTL);
-		free(data);
-		__leave;
-		return NULL;
-	}
-
-	bread= usb_pipe_read(njb, data, size+5);
-	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
-		free(data);
-		__leave;
-		return NULL;
-	} else if ( bread < size ) {
-		NJB_ERROR(EO_RDSHORT);
-		free(data);
-		__leave;
-		return NULL;
-	}
-
-	if ( data[0] ) {
-		NJB_STATUS((int) data[0]);
-		free(data);
-		__leave;
-		return NULL;
-	}
-
-	actsize = njb1_bytes_to_32bit(&data[1]);
-
-	/* Unpack the EAX structure */
-
-	eax= eax_unpack(&data[5], actsize);
-
-	free(data);
-
-	__leave;
-	return eax;
-}
-
-/* This is the new EAX API */
+/**
+ * This reads in the block with EAX types. It is a byte
+ * chunk which is then interpreted and the EAX effects are
+ * added to a list of effects which is then retrieved by
+ * calls to the <code>njb_get_nexteax()</code> function.
+ * 
+ * @param njb a pointer to the NJB object to use
+ * @param size the previously determined size of the EAX block
+ */
 void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
 {
   __dsub= "njb_read_eaxtypes";
@@ -1044,12 +1155,11 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   ssize_t bread;
   u_int32_t actsize;
   unsigned char usize[4];
-
   __enter;
 
   data = (unsigned char *) malloc(size);
   if ( data == NULL ) {
-    NJB_ERROR(EO_NOMEM);
+    NJB_ERROR(njb, EO_NOMEM);
     __leave;
     return;
   }
@@ -1061,7 +1171,7 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, NJB_CMD_GET_EAX,
 		 0, 0, 4, usize) == -1 ) {
     
-    NJB_ERROR(EO_USBCTL);
+    NJB_ERROR(njb, EO_USBCTL);
     free(data);
     __leave;
     return;
@@ -1069,19 +1179,19 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
 
   bread = usb_pipe_read(njb, data, size+5);
   if ( bread == -1 ) {
-    NJB_ERROR(EO_USBBLK);
+    NJB_ERROR(njb, EO_USBBLK);
     free(data);
     __leave;
     return;
   } else if ( bread < size ) {
-    NJB_ERROR(EO_RDSHORT);
+    NJB_ERROR(njb, EO_RDSHORT);
     free(data);
     __leave;
     return;
   }
 
   if ( data[0] ) {
-    NJB_STATUS((int) data[0]);
+    NJB_STATUS(njb,(int) data[0]);
     free(data);
     __leave;
     return;
@@ -1090,7 +1200,7 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   actsize = njb1_bytes_to_32bit(&data[1]);
 
   /* Unpack the EAX structure */
-  eax_unpack_new_api(&data[5], actsize, state);
+  eax_unpack(&data[5], actsize, state);
 
   free(data);
 
@@ -1098,6 +1208,12 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   return;
 }
 
+/**
+ * This retrieves an EAX effect from the list of
+ * effects read in by the previous functions.
+ *
+ * @param njb a pointer to the NJB object to use
+ */
 njb_eax_t *njb_get_nexteax(njb_t *njb)
 {
   njb_state_t *state = (njb_state_t *) njb->protocol_state;
@@ -1106,12 +1222,25 @@ njb_eax_t *njb_get_nexteax(njb_t *njb)
   ret = state->next_eax;
   if (ret != NULL) {
     state->next_eax = ret->next;
+  } else {
+    /* 
+     * When we have returned the last EAX type, make sure
+     * there are no dangerous pointers into the void
+     */
+    state->first_eax = NULL;
   }
   return ret;
 }
 
-/* This routine is dangerous. Do not call it... */
-int njb_refresh_eax(njb_t *njb, eax_t *eax)
+/*
+ * This routine is dangerous. At one time, the NJB1 would
+ * use this to update the status of the EAX settings, but
+ * with newer firmware it has become increasingly unreliable.
+ * So we simply do not use it, re-read the EAX types instead. 
+ */
+
+/*
+int njb_refresh_eax(njb_t *njb)
 {
 	__dsub= "njb_refresh_eax";
 	unsigned char *data;
@@ -1122,7 +1251,7 @@ int njb_refresh_eax(njb_t *njb, eax_t *eax)
 
 	data= malloc(4);
 	if (data == NULL) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -1130,42 +1259,42 @@ int njb_refresh_eax(njb_t *njb, eax_t *eax)
 	
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 			NJB_CMD_GET_EAX, 1, 0, 4, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return -1;
 	}
 
-	/* In newer firmware, the four bytes returned in data
-	 * contain rubbish, and can not be used for determining
-	 * the size of the EAX parameters. You have to discard it. */
+	// In newer firmware, the four bytes returned in data
+	// contain rubbish, and can not be used for determining
+	// the size of the EAX parameters. You have to discard it.
 	free(data);
 
-	/* 15 bytes + Status byte + 4 bytes size (again) */
+	// 15 bytes + Status byte + 4 bytes size (again)
 	data= malloc(20);
 	if (data == NULL) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
 
-	/* Read from bulk */
+	// Read from bulk
 	if ( (bread= usb_pipe_read(njb, data, actsize+5))
 		== -1 ) {
 
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return -1;
 	} else if ( bread < actsize+5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		free(data);
 		__leave;
 		return -1;
 	}
 	
 	if ( data[0] ) {
-		NJB_ERROR(EO_BADSTATUS);
+		NJB_ERROR(njb, EO_BADSTATUS);
 		free(data);
 		__leave;
 		return -1;
@@ -1179,6 +1308,7 @@ int njb_refresh_eax(njb_t *njb, eax_t *eax)
 	__leave;
 	return 0;
 }
+*/
 
 njb_time_t *njb_get_time(njb_t *njb)
 {
@@ -1190,7 +1320,7 @@ njb_time_t *njb_get_time(njb_t *njb)
 
 	data= malloc(17);
 	if (data == NULL) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -1198,18 +1328,18 @@ njb_time_t *njb_get_time(njb_t *njb)
 	
        	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
        			NJB_CMD_GET_TIME, 0, 0, 17, data) == -1 ) {
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
        		__leave;
        		return NULL;
        	}
        	if ( data[0] ) {
-       		NJB_ERROR(EO_BADSTATUS);
+       		NJB_ERROR(njb, EO_BADSTATUS);
 		free(data);
        		__leave;
        		return NULL;
        	}
-       	time= time_unpack(&data[1]);
+       	time = time_unpack(&data[1]);
 	
 	free(data);
 
@@ -1221,6 +1351,8 @@ int njb_set_time(njb_t *njb, njb_time_t *time)
 {
 	__dsub= "njb_set_time";
 	unsigned char *data;
+	
+	__enter;
 
        	data= time_pack(time);
 
@@ -1232,7 +1364,7 @@ int njb_set_time(njb_t *njb, njb_time_t *time)
        	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_SET_TIME, 0, 0, 16, data) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        	free(data);
        		__leave;
        		return -1;
@@ -1254,7 +1386,7 @@ int njb_create_playlist (njb_t *njb, char *name, u_int32_t *plid)
 
 	size= strlen(name) + 1;
 	if ( size > 0xffffffff ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return 0;
 	}
@@ -1266,35 +1398,35 @@ int njb_create_playlist (njb_t *njb, char *name, u_int32_t *plid)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_CREATE_PLAYLIST, lsw, msw, 0, NULL) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	bwritten= usb_pipe_write(njb, name, size);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < size ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
 
 	bread= usb_pipe_read(njb, retdata, 5);
 	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bread < 5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		__leave;
 		return -1;
 	}
 
 	if ( retdata[0] ) {
-		NJB_STATUS(retdata[0]);
+		NJB_STATUS(njb,retdata[0]);
 		__leave;
 		return -1;
 	}
@@ -1319,13 +1451,13 @@ int njb_delete_playlist (njb_t *njb, u_int32_t plid)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_PLAYLIST, msw, lsw, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -1347,7 +1479,7 @@ int njb_rename_playlist (njb_t *njb, u_int32_t plid, char *name)
 	
 	size= strlen(name) + 1;
 	if ( size > 0xffffffff ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return 0;
 	}
@@ -1358,18 +1490,18 @@ int njb_rename_playlist (njb_t *njb, u_int32_t plid, char *name)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_RENAME_PLAYLIST, 0, 0, 8, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	bwritten= usb_pipe_write(njb, name, size);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < size ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
@@ -1392,7 +1524,7 @@ int njb_add_track_to_playlist (njb_t *njb, u_int32_t plid, u_int32_t trid)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_ADD_TRACK_TO_PLAYLIST, 0, 0, 10, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -1416,7 +1548,7 @@ int njb_add_multiple_tracks_to_playlist (njb_t *njb, u_int32_t plid,
 	bsize= ntracks*6;
 	block= (unsigned char *) malloc(bsize);
 	if ( block == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -1438,7 +1570,7 @@ int njb_add_multiple_tracks_to_playlist (njb_t *njb, u_int32_t plid,
 		NJB_CMD_ADD_MULTIPLE_TRACKS_TO_PLAYLIST, 0, 0, 6, data) 
 		== -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(block);
 		__leave;
 		return -1;
@@ -1446,12 +1578,12 @@ int njb_add_multiple_tracks_to_playlist (njb_t *njb, u_int32_t plid,
 
 	bwritten= usb_pipe_write(njb, block, bsize);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(block);
 		__leave;
 		return -1;
 	} else if ( bwritten < bsize ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		free(block);
 		__leave;
 		return -1;
@@ -1476,13 +1608,13 @@ int njb_delete_track (njb_t *njb, u_int32_t trackid)
 
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_TRACK, msw, lsw, 1, &data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -1504,13 +1636,13 @@ int njb_delete_datafile (njb_t *njb, u_int32_t fileid)
 
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_DATAFILE, msw, lsw, 1, &data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -1533,7 +1665,7 @@ int njb_play_or_queue (njb_t *njb, u_int32_t trackid, int cmd)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, cmd, 0, 0, 4, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -1553,7 +1685,7 @@ int njb_elapsed_time (njb_t *njb, u_int16_t *elapsed, int *change)
 	
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_ELAPSED_TIME, 0, 0, 3, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -1563,7 +1695,7 @@ int njb_elapsed_time (njb_t *njb, u_int16_t *elapsed, int *change)
 	} else if ( data[0] == 0 ) {
 		*change= 0;
 	} else {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -1576,19 +1708,19 @@ int njb_elapsed_time (njb_t *njb, u_int16_t *elapsed, int *change)
 
 int njb_replace_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 {
-	__dsub= "njb_replace_track_tag";
+	__dsub = "njb_replace_track_tag";
 	u_int16_t msw, lsw;
 	unsigned char *data;
 	ssize_t bwritten;
 
 	__enter;
 
-	msw= get_msw(tagh->size);
-	lsw= get_lsw(tagh->size);
+	msw = get_msw(tagh->size);
+	lsw = get_lsw(tagh->size);
 
-	data= (unsigned char *) malloc(tagh->size+4);
+	data = (unsigned char *) malloc(tagh->size+4);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -1599,21 +1731,20 @@ int njb_replace_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_REPLACE_TRACK_TAG, lsw, msw, 0, NULL) == -1 ) {
-
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return -1;
 	}
 
-	bwritten= usb_pipe_write(njb, data, tagh->size+4);
+	bwritten = usb_pipe_write(njb, data, tagh->size+4);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		free(data);
 		return -1;
 	} else if ( bwritten < (tagh->size+4) ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		free(data);
 		__leave;
 		return -1;
@@ -1635,13 +1766,13 @@ int njb_adjust_sound(njb_t *njb, u_int8_t effect, int16_t value)
 	memset(data, 0, 3);
 	data[0] = effect;
 
-	printf("Effect %d, sending value %d (%04X)\n", effect, sendvalue, sendvalue);
+	/* printf("Effect %d, sending value %d (%04X)\n", effect, sendvalue, sendvalue); */
 
 	from_16bit_to_njb1_bytes(sendvalue, &data[1]);
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_ADJUST_SOUND, 0, 0, 3, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
